

//Accessors control all the data of the football for ownership.
GetBallData( datatype )
{
	if(datatype < BALL_COUNT)
		return g_iBallData[datatype];
	return -1;
}

SetBallData( datatype, newValue )
{
	if(datatype < BALL_COUNT)
		g_iBallData[datatype] = newValue;	
}

public ThinkFootball( football )
{
	new flytype = GetBallData( BALL_FLY );
	new spintype = GetBallData( BALL_SPIN );
	
	
	
	//Run our custom movement/spin stuff.
	Football_Movement( football, flytype );
	Football_Spin( football, spintype );
	
	//Track our ball position and determine if we passed line of scrimmage
	Football_Track();

	Handle_Scrimmage();
	
	//Always thinking!
	entity_set_float( football, EV_FL_nextthink, get_gametime() + 0.01 );
}

bool:Handle_Throw( grenade )
{
	new bool:inPlay = (g_iGameMode == GAME_INPLAY);
	
	if(!inPlay && g_bGameStart)//g_iGameMode != GAME_NONE)
		return false;
		
	//if(!is_user_alive(id))
	//	return false;
		
	//Are we allowed to even throw a football?
	new id = entity_get_edict( grenade, EV_ENT_owner );
	new holder = GetBallData(BALL_HOLDER);
	
	if( id != holder )
		return false;
		
	//Get football fly type
	new flytype = FLY_LATERAL;
	new passtype = PASS_BULLET;
	new throwdir = PlayerDirection(id);
	
	new bool:isFacingForward = ( throwdir == _FORWARD );
			
	//Only throw forward if behind scim
	if( isFacingForward )
	{
		if(Ability_IsActive( id, A_DIVE ) || (inPlay && g_bPassedScrimmage) || g_bPlayClockOn)
		{
			Football_Give( id, false );
			return false;
		}
			
		flytype = FLY_PASS;
		
		
		//If we're aiming high make it a bomb pass
		new Float:angle[3];
		entity_get_vector(id, EV_VEC_v_angle, angle);
	
		if(angle[0] < -20.0)
			passtype = PASS_BOMB;
			
		SetBallData( BALL_PASS, passtype );
		
		STATISTIC( id, S_PASS_ATTEMPT, 1 );
		
		SetBallData( BALL_THROW_PLAYER, id );
		SetBallData( BALL_THROW_FLY, flytype );
	}
	
	//Saves our ball flying data
	SetBallData( BALL_FLY, flytype );
	
	//Throw the actual ball.
	Football_Throw( grenade, flytype );
	
	if(inPlay)
	{
		g_bTouchedGround = false;
		g_flStartYard[GetBallData(BALL_START_TEAM)] = g_flCurrentBallYard;
		//Stat_ThrowBall(id);
	}
	
	return true;
}

bool:Handle_Kick(player)
{
	new flytype = GetBallData( BALL_FLY );
	
	//Ball must be in kick mode.
	if(flytype != FLY_KICK_HOLD && flytype != FLY_FG_HOLD)
		return false;
		
	//new ballTeam = GetBallData( BALL_TEAM );
	new class = GetClass(player);
	//We must be Kicker and Ball must be in Kick Off or Field Goal stand mode
	if(class != CLASS_KICKER)
		return false;
		
	new startTeam = GetBallData( BALL_START_TEAM );
	new isKickoff = (flytype == FLY_KICK_HOLD);
	
	//We must kick forward
	new dir = PlayerDirection( player );
	if(dir == _FORWARD)
	{
		new flytype = FLY_FG;
		if(isKickoff)
		{
			UnfreezeAll();
			flytype = FLY_KICK;
			STATISTIC( player, S_KICK_ATTEMPT, 1 );
		}
		else
		{
			new dist = floatround(YARD_FG_POST - g_flCurrentYard);
			if(dist > 50)
				STATISTIC( player, S_FG_ATTEMPT_50, 1 );	
			else if(dist > 40)
				STATISTIC( player, S_FG_ATTEMPT_40, 1 );	
			else if(dist > 30)
				STATISTIC( player, S_FG_ATTEMPT_30, 1 );	
			else if(dist > 20)
				STATISTIC( player, S_FG_ATTEMPT_20, 1 );	
			else
				STATISTIC( player, S_FG_ATTEMPT_0, 1 );	
		}
		Football_Kick( player, flytype );	
		
		g_flStartYard[startTeam] = g_flCurrentBallYard;
		g_bTouchedGround = false;
		
		StopPlayClock();
		if(!g_bExtraPoint)
			StartClock();
		
		return true;	
	}
	return false;
}


bool:Handle_Punt(player)
{
	if(g_iGameMode != GAME_INPLAY)
		return false;

	new team = get_user_team(player);
	if(team < 1 || team > 2 || GetDefaultClass(player) == 0)
		return false;
				
	new flytype = GetBallData( BALL_FLY );
	new holder = GetBallData( BALL_HOLDER );
	new startTeam = GetBallData( BALL_START_TEAM );
	
	if(!g_bPlayClockOn && flytype == FLY_PUNT_HOLD)
	{
		//If we're punter handle kicking ball
		if( holder == player )
		{
			new buttons = entity_get_int( player, EV_INT_button );
			new oldbuttons = entity_get_int( player, EV_INT_oldbuttons );
			if((buttons & IN_ATTACK) && !(oldbuttons & IN_ATTACK) )
			{
				STATISTIC( player, S_PUNT_ATTEMPT, 1 );
				
				Football_Kick( player, FLY_PUNT );
				
				g_bTouchedGround = false;
				g_flStartYard[GetBallData(BALL_START_TEAM)] = g_flCurrentBallYard;
			}
		}
		//If punter has not kicked don't let team except receivers pass line of scrimmage
		else if( team == startTeam && GetClass(player) != CLASS_RECEIVER )
		{
			new Float:scrimYard = GetOriginYard( g_flCurrentYard );
			new Float:origin[3];
			new Float:playerYard;
			
			entity_get_vector(player,EV_VEC_origin, origin );
			playerYard = GetYard( origin[1] );
			
			if(playerYard > g_flCurrentYard)
				origin[1] = scrimYard;
				
			entity_set_vector(player,EV_VEC_origin, origin );
		}	
	}
	return false;
}

bool:Handle_Handoff(id)
{
	if(g_iGameMode != GAME_INPLAY)
		return false;
		
	new ballholder = GetBallData( BALL_HOLDER );
	if(ballholder != id || !is_user_alive(ballholder))
		return false;
	
	//If we passed scrimmage, we can't handoff
	new flytype = GetBallData( BALL_FLY );
	if( g_bPassedScrimmage )	
	{
		if(flytype != FLY_NONE)
		{
			Football_Give( id, false );
		}
		return true;
	}
		
	new buttons = entity_get_int(id, EV_INT_button);
	new oldbuttons = entity_get_int(id, EV_INT_oldbuttons);
	new bool:bUseKey = (buttons & IN_USE) > 0;//;
	new bool:bReleasedUseKey = (!(buttons & IN_USE) && (oldbuttons & IN_USE));
	//new bool:bPlayClock = IsPlayClockOn();
	//new bool:bAttackKey = (!(buttons & IN_ATTACK) && (oldbuttons & IN_ATTACK));
	
	if(g_iPlay == PLAY_PUNT)
	{
		//Punter can fake punt only after he Huts, and cannot go back to punt mode.
		if(	bReleasedUseKey && !g_bPlayClockOn && g_iPlay == PLAY_PUNT && flytype == FLY_PUNT_HOLD )
		{
			g_iPlay = PLAY_STANDARD;
			Football_PuntHold(id, false);
		}
	}
	else
	{
		if(bUseKey)
		{
			if(flytype != FLY_HANDOFF)
				Football_Handoff(id, true);	
		}
		else
		{
			if(flytype != FLY_NONE)
				Football_Handoff(id, false);
		}
	}
	
	/*
	//Should we allow handoff pitching?
	if(bAttackKey)
	{
		if(flytype == FLY_HANDOFF)
		{
			new pDir = PlayerDirection(id);
			if(pDir == _BACK)
			{
				SetBallData( BALL_FLY, FLY_PITCH );
				Football_Throw( FLY_PITCH );
			}
		}	
	}
	*/
	
	return false;
}

Football_Track()
{
	if(g_iGameMode != GAME_INPLAY)
		return;
		
	//only track ball if its in-bounds.
	new flytype = GetBallData( BALL_FLY );
	if(flytype == FLY_DEAD)
		return;
			
	if(flytype != FLY_NONE)
	{
		//Tracking ball.
		//if( flytype != FLY_PASS && g_iBounds[0] != OUTOFBOUND )
		if( g_iBounds[0] != OUTOFBOUND )
			MarkBallYard( GetBallData( BALL_ID ) );
	}
	else
	{
		//Tracking player holding ball.
		new holder = GetBallData( BALL_HOLDER );
		if( g_iBounds[holder] != OUTOFBOUND )
			MarkBallYard( holder );
	}	
}

MarkBallYard( entity )
{
	entity_get_vector(entity, EV_VEC_origin, g_vecBall );
	g_flCurrentBallYard = GetYard( g_vecBall[1] );
}

Football_Movement( football, flytype )
{
	switch( flytype )
	{
		case FLY_PASS:
		{
			new Float:vel[3];
			entity_get_vector( football, EV_VEC_velocity, vel );
			new passType = GetBallData( BALL_PASS );
			if(passType == PASS_BOMB)
			{
				if(vel[2] < 0.0)
					entity_set_float( football, EV_FL_gravity, 0.6 );	
			
					
				//Increase gravity on the way down for bomb throw
				
				//if(vel[2] < 0.0)
				//	entity_set_float( football, EV_FL_gravity, 0.75 );	
			}
			else if(passType == PASS_BULLET)
			{
				//Convert back to float by .000 precision
				//if(vel[2] < 0.0)
				//	entity_set_float( football, EV_FL_gravity, 0.5 );
			}	
		}
	}
}

Football_Spin( football, spintype )
{
	new Float:angle[3];
	new Float:velocity[3];
	
	switch(spintype)
	{
		case SPIN_NONE:
		{
			new holder = GetBallData( BALL_HOLDER );
			new Float:origin[3];
			entity_get_vector(holder, EV_VEC_origin, origin );
			entity_set_vector(football, EV_VEC_origin, origin );
		}
		case SPIN_SPIRAL:
		{
			//Roll along the Z axis
			new Float:roll;
			entity_get_vector(football,EV_VEC_velocity,velocity);
			entity_get_vector(football,EV_VEC_angles,angle);
			roll = angle[2];
	
			vector_to_angle(velocity,angle);
			
			new Float:frametime;
			global_get(glb_frametime, frametime );
			new Float:gametime = get_gametime();
			new owner = GetBallData( BALL_OWNER );
			new class = GetClass( owner );
			if( class != CLASS_QUARTERBACK )
			{
				angle[0] += 30.0 * floatcos( gametime * 10.0 );
				angle[1] += 30.0 * floatcos( gametime * 10.0 );
			}
			angle[2] = roll + ( 540.0 * frametime );
			entity_set_vector(football, EV_VEC_angles, angle );
		}
		case SPIN_VERTICAL:
		{
			new id = GetBallData( BALL_OWNER );//entity_get_edict(football, EV_ENT_owner);
			new Float:origin[3];
			entity_get_vector(id, EV_VEC_origin, origin );
			entity_get_vector(id, EV_VEC_angles, angle );
			angle[0] = 0.0;
			angle[2] = 0.0;
			angle_vector(angle, ANGLEVECTOR_FORWARD, velocity);
			origin[0] += velocity[0] * 50.0;
			origin[1] += velocity[1] * 50.0;
			entity_set_origin(football, origin );
				
			new Float:pAng[3];
			entity_get_vector(id, EV_VEC_angles, pAng);
			entity_get_vector(football, EV_VEC_angles, angle );
			angle[0] = 90.0;
			angle[1] = 0.0;
			angle[2] = pAng[1];
			entity_set_vector(football, EV_VEC_angles, angle );
		}
		case SPIN_HORIZONTAL:
		{
			new id = GetBallData( BALL_HOLDER );//entity_get_edict(football, EV_ENT_owner);
			new Float:origin[3];
			entity_get_vector(id, EV_VEC_origin, origin );
			entity_get_vector(id, EV_VEC_angles, angle );
			angle_vector(angle, ANGLEVECTOR_FORWARD, velocity);
			origin[0] += velocity[0] * 50.0;
			origin[1] += velocity[1] * 50.0;
			entity_set_origin(football, origin );
				
			new Float:pAng[3];
			entity_get_vector(id, EV_VEC_angles, pAng);
			entity_get_vector(football, EV_VEC_angles, angle );
			angle[0] = 0.0;//pAng[0];
			angle[1] = pAng[1];
			angle[2] = 0.0;//pAng[1];
			entity_set_vector(football, EV_VEC_angles, angle );
		}
		case SPIN_WOBBLE_SLOW, SPIN_WOBBLE:
		{
			entity_get_vector(football,EV_VEC_velocity,velocity);
			//does what vector_length does, but negates the square root which is EXTREMELY slow. 640,000 = 800 units squared
			new Float:speed = ((velocity[0] * velocity[0]) + (velocity[1]*velocity[1]) + (velocity[2]*velocity[2])) / 250000.0;
		
			if( spintype == SPIN_WOBBLE_SLOW )
				speed *= 0.2;
				
			entity_get_vector(football,EV_VEC_angles,angle);
	
			angle[0] += 15.0 * speed;
			angle[1] += 12.0 * speed;
			angle[2] += 8.0 * speed;
			entity_set_vector(football, EV_VEC_angles, angle );
		}
		
	}
}

Football_Throw( grenade, flytype )
{			
	Football_Hide(false);
	g_flFootballTossTime = get_gametime();
	//Create the actual ball
	new ball = GetBallData(BALL_ID);//Football_Create();
	new holder = GetBallData( BALL_HOLDER );
	//new team = get_user_team(holder);
	//new startTeam = GetBallData( BALL_START_TEAM );
	MarkBallYard( ball );
	
	SetBallData( BALL_OWNER, holder );
	SetBallData( BALL_HOLDER, 0 );
	//SetBallData( BALL_TEAM, team );
	
	if(g_iGameMode != GAME_INPLAY && g_bGameStart)
		Football_Dead( g_flCurrentYard );
		
	//Adjust our velocities for the throw
	new Float:origin[3], Float:vel[3], Float:angle[3];
	entity_get_vector(holder,EV_VEC_origin,origin);
	entity_get_vector(grenade,EV_VEC_velocity,vel);
	entity_get_vector(holder, EV_VEC_v_angle, angle );
	entity_set_float( ball, EV_FL_gravity, BALL_GRAVITY );
	
	new passType = GetBallData( BALL_PASS );
	switch( flytype )
	{
		case FLY_PASS:
		{
			SetBallData( BALL_SPIN, SPIN_SPIRAL );
			
			if(passType == PASS_BOMB)
			{
				//ORIGINALS
				vel[0] *= 0.9;
				vel[1] *= 0.9;
					
				entity_set_float( ball, EV_FL_gravity, BALL_GRAVITY-0.1 );
			}
			else
			{
				//ORIGINALS
				vel[0] *= 1.28;
				vel[1] *= 1.28;
				vel[2] += 75.0;
			}
	
			FX_Trail(ball, 3);
			FX_Trail(ball, 2);
		}
		case FLY_LATERAL:
		{
			SetBallData( BALL_SPIN, SPIN_WOBBLE );

			vel[0] *= 0.68;
			vel[1] *= 0.68;
			if(vel[2] > 300.0)
				vel[2] = 300.0;
		}
		case FLY_PITCH:
		{
			SetBallData( BALL_SPIN, SPIN_WOBBLE_SLOW );
		
			velocity_by_aim(holder, 400, vel);
			vel[2] = 0.0;
		}
	}

	//Hack to fix exploit throwing the opposite direction
	//new throwdir = (flytype == FLY_LATERAL) ? -1 : 1;
	//if( CheckVelocityDirection( team, vel ) != throwdir )
	//{
	//	vel[0] *= -1.0;
	//	vel[1] *= -1.0;	
	//}
	
	origin[2] += 16.0;
	entity_set_origin(ball, origin );
	entity_set_vector(ball,EV_VEC_velocity,vel);
	
	new Float:startOrig[3];
	startOrig[0] = origin[0];
	startOrig[1] = origin[1];
	startOrig[2] = origin[2];
	
	new Float:startVel;
	startVel = vel[2];
	
	//Predict the landing and save to our global g_vLandOrig
	new Float:startYard = GetYard( origin[1] );
	
	new Float:dex = Ability_Strength(holder, A_THROW);
			
	PredictLanding( ball, origin, vel, (flytype == FLY_PASS && passType == PASS_BOMB) );
	new Float:endYard = GetYard(origin[1] );
	while( endYard - startYard > (70.0*dex) )
	{
		vel[0] *= 0.85;
		vel[1] *= 0.85;
		
		origin[0] = startOrig[0];
		origin[1] = startOrig[1];
		origin[2] = startOrig[2];
		
		vel[2] = startVel;
		
		PredictLanding( ball, origin, vel, (flytype == FLY_PASS && passType == PASS_BOMB) );
		endYard = GetYard(origin[1]);
		
		
		//origin[2] += 10.0;
	}
	
	vel[2] = startVel;
	entity_set_vector(ball,EV_VEC_velocity,vel);
	
	if(flytype == FLY_PASS)
		client_print(holder, print_chat, "Threw %i yards.", floatround(endYard-startYard) );
	FVecIVec( origin, g_vLandOrig );
	
	entity_set_string(holder, EV_SZ_viewmodel, "models/v_bknuckles.mdl");
	entity_set_string(holder, EV_SZ_weaponmodel, "models/p_bknuckles.mdl")
	
}

//Get the players current view angle, convert it to a vector direction, 
// multiply our speed into it, and set it as the balls velocity.
Football_Kick( id, flytype )
{
	if(g_iGameMode != GAME_INPLAY)
		return;

	Football_Hide(false);		
	g_flFootballTossTime = get_gametime();
	
	SetBallData( BALL_HOLDER, 0 );
	SetBallData( BALL_OWNER, id );
	SetBallData( BALL_SPIN, SPIN_WOBBLE );
	SetBallData( BALL_FLY, flytype );
	
	new Float:vel[3], Float:aimVel[3], Float:angle[3];
	new Float:speed, Float:playerSpeed;//, Float:pitch, Float:upVel, Float:vForward[3];
	new football = GetBallData( BALL_ID );
	new onside;
	MarkBallYard( football );
	
		
	entity_get_vector(id, EV_VEC_velocity, vel );
	entity_get_vector(id, EV_VEC_v_angle, angle );
	
	speed = vector_length( vel );
	
	switch(flytype)
	{
		case FLY_FG:
		{
			if(angle[0] > -10.0)
				angle[0] = -10.0;
			else if(angle[0] < -40.0)
				angle[0] = -40.0;
				
			angle[0] -= 30.0;
			speed = 800.0;
			
			if( on_ground(id) )
			{
				vel[0] *= 1.2;
				vel[1] *= 1.2;
				angle[0] -= 10.0;
			}
		}
		case FLY_PUNT:
		{
			Give( id, "weapon_knife" );

			SetBallData( BALL_SPIN, SPIN_WOBBLE_SLOW );
			
			vel[0] *= 0.15;
			vel[1] *= 0.15;
			speed = 1000.0;
			angle[0] = -60.0;	
		}
		case FLY_KICK:
		{
			if(angle[0] < -10.0)
			{
				vel[0] *= 1.1;
				vel[1] *= 1.1;
				speed += 275.0;
				angle[0] = -60.0;
			}
			else
			{
				onside = 1;
				vel[2] = 0.0;
				speed += random_float( 200.0, 300.0 );
				angle[0] = -10.0;
			}
		}
	}

	angle_vector(angle, ANGLEVECTOR_FORWARD, aimVel);
	
	aimVel[0] = (speed * aimVel[0]) + vel[0];
	aimVel[1] = (speed * aimVel[1]) + vel[1];
	aimVel[2] = (speed * aimVel[2]) + vel[2];
	
	
	entity_set_vector(football, EV_VEC_velocity, aimVel );
	entity_get_vector(football, EV_VEC_origin, vel );
		
	///if(flytype == FLY_FG)
	FX_Trail(football);
	//else
	{
		new Float:startYard = GetYard(vel[1]);
		
		PredictLanding( football, vel, aimVel );
		FVecIVec( vel, g_vLandOrig );
		new Float:endYard = GetYard(vel[1]);
		
		if(flytype != FLY_FG)
			client_print(id, print_chat, "Kicked %i yards.", floatround(endYard-startYard) );
		
		new Float:yaw = floatabs(angle[1]);
		if( (onside && (yaw < 60.0 || yaw > 130.0)) || (GetYard(vel[0]) < 55.0 && playerSpeed < 150.0) )
			STATISTIC( id, S_KICK_ONSIDE_ATTEMPT, 1 );	
	}
		
	new Float:MinBox[3], Float:MaxBox[3];
	GetBallMinMax( MinBox, MaxBox );
	entity_set_size( football, MinBox, MaxBox );
}

Football_Spawn( flytype )//Float:origin[3]={0.0,0.0,0.0}, Float:angle[3]={0.0,0.0,0.0} )
{
	new Float:origin[3], Float:ang[3];
	new Float:yard = GetOriginYard( g_flCurrentYard );
	new dir = GetBallData( BALL_DIRECTION );
	ang[0] = 90.0;
	ang[1] = 90.0 * float(dir);
			
	if( flytype == FLY_FG_HOLD )
	{
		GetBallOrigin( origin, FIELDGOAL_BALLPOS );
		origin[1] = yard + (origin[1] * dir);
	}
	else
	{
		GetBallOrigin( origin, KICKOFF_BALLPOS );
		origin[1] = yard + (origin[1] * -dir);
	}
	
	new ball = GetBallData(BALL_ID);//Football_Create();
	
	if(ball)
	{
		//drop_to_floor(ball);
		
		new Float:vel[3];
		entity_set_vector(ball, EV_VEC_velocity, vel );
		entity_set_origin(ball, origin );
		entity_set_vector(ball, EV_VEC_angles, ang );
		
		new Float:mins[3], Float:maxs[3];
		GetBallMinMax( mins, maxs );
		maxs[2] = 128.0;
		entity_set_size( ball, mins, maxs );
	}
	
	Football_Clear(false);
	Football_Hide(false);
	
	SetBallData( BALL_FLY, flytype );
	
	new Float:vel[3];
	entity_set_vector(ball, EV_VEC_velocity, vel );
	return ball;
}

Football_Dead( Float:yard, bool:moveToLine=true )
{
	//new holder = GetBallData(BALL_HOLDER);

	if(!moveToLine)
	{
		
		new Float:origin[3];
		new Float:vel[3];
		new ball = GetBallData( BALL_ID );//Football_Create();
		
		if(ball)
		{
			origin[1] = GetOriginYard( yard );
			origin[2] = g_vSideLine[0][2] + 15;
			entity_set_origin(ball, origin );
			entity_set_vector(ball, EV_VEC_velocity, vel );
		}
		
		Football_Clear( false );
		Football_Hide( false );	
	}
	
	
	SetBallData( BALL_SPIN, SPIN_WOBBLE );
	
	
	//SetBallData( BALL_HOLDER, 0 );
	//SetBallData( BALL_OWNER, 0 );
	SetBallData( BALL_FLY, FLY_DEAD );
}

//Creates a football and sets it's owner and team.
Football_Create()
{	
	new ball = GetBallData( BALL_ID );

	if(ball == 0)
	{
		ball = create_entity("info_target");
		entity_set_string(ball,EV_SZ_classname,"football");
		
		entity_set_model(ball,"models/football/w_football2.mdl");
		
		entity_set_int( ball, EV_INT_solid, SOLID_TRIGGER);
		entity_set_int( ball, EV_INT_movetype, MOVETYPE_BOUNCE );
		//DispatchSpawn(ball);
		
		new Float:MinBox[3], Float:MaxBox[3];
		GetBallMinMax( MinBox, MaxBox );
		entity_set_size( ball, MinBox, MaxBox );
		
		new Float:origin[3];
		entity_set_origin(ball, origin );
		entity_set_float( ball, EV_FL_gravity, BALL_GRAVITY );
	
		SetBallData( BALL_ID, ball );
		
		Football_Fumble();
		
		entity_set_float( ball, EV_FL_nextthink, get_gametime() );
	}
	
	//entity_set_vector(ball, EV_VEC_mins, MinBox);
	//entity_set_vector(ball, EV_VEC_maxs, MaxBox);
	return ball;
}

Football_Fumble()
{
	
	new Float:origin[3], Float:vel[3];
	new id = GetBallData( BALL_HOLDER );
	new football = GetBallData( BALL_ID );
	//new flytype = GetBallData( BALL_FLY );
	
	if(id)
	{
		Football_Clear();
		
		entity_get_vector( id, EV_VEC_origin, origin );
		entity_set_origin( football, origin );
		//entity_set_int( football, EV_INT_movetype, MOVETYPE_BOUNCE );
		
		vel[0] = random_float( -300.0, 300.0 );
		vel[1] = random_float( -300.0, 300.0 );
		vel[2] = random_float( 50.0, 300.0 );
		
		Football_Hide(false);
		entity_set_vector( football, EV_VEC_velocity, vel );
		
		STATISTIC( id, S_FUMBLE, 1 );
	}
	
	SetBallData( BALL_FUMBLE, id );
	SetBallData( BALL_FLY, FLY_FUMBLE );
	SetBallData( BALL_SPIN, SPIN_WOBBLE );
	SetBallData( BALL_OWNER, id );
	SetBallData( BALL_HOLDER, 0 );
}

Football_Give( id, bool:delayed=true )
{
	if(is_user_alive(id))
	{
		new flytype = GetBallData(BALL_FLY);
		if(GetBallData(BALL_HOLDER) != id || (flytype == FLY_PUNT_HOLD || flytype == FLY_HANDOFF))
			Football_Clear( true );
		
		SetBallData( BALL_HOLDER, id );
		SetBallData( BALL_OWNER, id );
		
		SetBallData( BALL_FLY, FLY_NONE );
		SetBallData( BALL_SPIN, SPIN_NONE );
		//new team =  get_user_team(id);
		//SetBallData( BALL_TEAM, team );
	
		if(delayed)
			set_task(0.1, "GiveGrenade", id );
		else
			GiveGrenade(id);
	}
}
public GiveGrenade(id)
{
	if(!user_has_weapon(id, CSW_HEGRENADE))
		Give( id, "weapon_hegrenade" );

	cs_set_user_bpammo(id, CSW_HEGRENADE, 1);
	entity_set_string(id,EV_SZ_weaponmodel,"models/football/p_football.mdl");	
}

Football_Handoff( id, bool:active )
{
	if(!active)
	{
		
		Give( id, "weapon_knife" );
		Football_Give( id, false );
	}
	else
	{
		Football_Hide(false);
		
		strip_user_weapons(id);
		g_iWeapon[id] = 0;
		cs_set_user_bpammo(id, CSW_HEGRENADE, 0);
		
		entity_set_string(id, EV_SZ_weaponmodel, "models/p_bknuckles.mdl")
		
		SetBallData( BALL_OWNER, id );
		SetBallData( BALL_SPIN, SPIN_VERTICAL );
		SetBallData( BALL_FLY, FLY_HANDOFF );
	}
}

Football_PuntHold( id, bool:active )
{
	if(!active)
	{
		Give( id, "weapon_knife" );
		Football_Give( id, false );
	}
	else
	{
		Football_Hide(false);
		
		Football_Clear( false );
			
		SetBallData( BALL_HOLDER, id );
		SetBallData( BALL_OWNER, id );
		
		strip_user_weapons(id);
		g_iWeapon[id] = 0;
		cs_set_user_bpammo(id, CSW_HEGRENADE, 0);
		
		entity_set_string(id, EV_SZ_weaponmodel, "models/p_bknuckles.mdl")
		SetBallData( BALL_SPIN, SPIN_HORIZONTAL );
		SetBallData( BALL_FLY, FLY_PUNT_HOLD );
	}
}

Football_Clear( bool:hide=true )
{
	new ballholder = GetBallData( BALL_HOLDER );

	if(is_user_alive(ballholder) && user_has_weapon(ballholder, CSW_HEGRENADE))
	{
		strip_user_weapons(ballholder);
		g_iWeapon[ballholder] = 0;
		cs_set_user_bpammo( ballholder, CSW_HEGRENADE, 0)
		//LastInv(ballholder);
		entity_set_string(ballholder, EV_SZ_weaponmodel, "models/p_bknuckles.mdl")
	}
	
	SetBallData( BALL_HOLDER, 0 );
		
	if(hide)
		Football_Hide( true );
}

Football_Hide( bool:visible )
{
	new football = GetBallData( BALL_ID );
	if(football == 0)
		return;
		
	new effects = entity_get_int( football, EV_INT_effects );
	if(!visible)
		entity_set_int( football, EV_INT_effects, effects & ~EF_NODRAW );
	else
		entity_set_int( football, EV_INT_effects, effects | EF_NODRAW );
}

Football_Bounce(football)
{
	new Float:vel[3];
	new flytype = GetBallData( BALL_FLY );
	
	if(flytype == FLY_KICK_HOLD || flytype == FLY_FG_HOLD || flytype == FLY_PUNT_HOLD || flytype == FLY_HANDOFF)
	{
		entity_set_vector(football, EV_VEC_velocity, vel );
		return;
	}
		
	SetBallData( BALL_SPIN, SPIN_WOBBLE );
	
	if(g_bLandEffect)
		g_bLandEffect = false;
		
	entity_get_vector(football, EV_VEC_velocity, vel );
	new Float:speed = vel[0]*vel[0] + vel[1]*vel[1];
	
	if(flytype == FLY_PASS)
	{
		vel[0] *= 0.3;
		vel[1] *= 0.3;
		vel[2] *= 0.7;	
	}
	if(speed < 10000) // 100 units a second (100^2)
	{
		vel[0] *= 0.7;
		vel[1] *= 0.7;
		vel[2] *= 0.6;	
	}
	else
	{
		new Float:speed = vector_length(vel) / 600.0;
		new Float:rand = random_float(0.5, 0.98)
		vel[0] *= rand;
		vel[1] *= rand;
		vel[2] = random_float((-300.0*speed), 50.0);	
	}
	
	entity_set_vector(football, EV_VEC_velocity, vel );
}

bool:Football_Deflect( player )
{
	new team = get_user_team(player);
	new startTeam = GetBallData( BALL_START_TEAM );
	new flytype = GetBallData(BALL_FLY);
	if(team == startTeam)
	{
		if(flytype != FLY_FG && flytype != FLY_PUNT)
			return false;	
	}
	
	new football = GetBallData( BALL_ID );
	new Float:orig[3];
	entity_get_vector(player, EV_VEC_origin, orig );
	new Float:yard = GetYard( orig[1] );
	new Float:curyard = g_flCurrentYard;
	new rand = random_num( 1, 100 );
	new needed = 100;
	
	if(team == startTeam && g_iPlay == PLAY_STANDARD && yard < curyard)
		needed -= 20;
	
	if(flytype == FLY_PASS)
		needed -= 10;
		
	if(flytype == FLY_FG)
		needed -= 30;
		
	if(flytype == FLY_PUNT)
		needed -= 30;
	
	if(rand > needed)
	{
		new Float:vel[3];
		new Float:pushVel[3];
		
		entity_get_vector(football, EV_VEC_velocity, vel );
		pushVel[0] = vel[0] * random_float(0.1,0.3);
		pushVel[1] = vel[1] * random_float(0.1,0.3);
		pushVel[2] = vel[2] + random_float(50.0,200.0);
		entity_set_vector(player,EV_VEC_velocity,pushVel);
	
		vel[0] = (vel[0] * random_float(0.1,0.5)) * -1.0;
		vel[1] = (vel[1] * random_float(0.1,0.5)) * -1.0;
		vel[2] = (200 * random_float(-2.0,-0.1));
				
		entity_set_vector(football, EV_VEC_velocity, vel);
		//ChangeBallState( BALL_BOUNCE );
		return true;
	}	
		
	return false;
}
PredictLanding( ball, Float:orig[3], Float:vel[3], bool:bomb=false )
{
	new Float:end[3], Float:start[3], Float:ret[3];
	new Float:gravity, Float:frametime = get_global_float(GL_frametime);

	ret[0] = orig[0];	ret[1] = orig[1];	ret[2] = orig[2];
	end[0] = orig[0];	end[1] = orig[1];	end[2] = orig[2];
	
	//slow our velocity down to one frame time;
	new Float:x = vel[0] * frametime;
	new Float:y = vel[1] * frametime;

	//get our gravity amount to lose per frametime
	new Float:serverGravity = get_cvar_float("sv_gravity") * frametime;
	
	new flytype = GetBallData( BALL_FLY );
	new passtype = GetBallData( BALL_PASS );
	new bool:isPass = (flytype == FLY_PASS);//flytype == FLY_BULLET || bomb);
	new Float:ballgravity = 0.0;
			
	if(isPass)
	{
		if(!bomb)
			ballgravity = BALL_GRAVITY;
		else
			ballgravity = BALL_GRAVITY-0.1;
	}
	else
		ballgravity = entity_get_float(ball, EV_FL_gravity);
		
	gravity = ballgravity * serverGravity;
	serverGravity = 0.6 * serverGravity;
	
	//This recreates the movement of a MOVETYPE_BOUNCE with X gravity and X velocity.
	//What it does is loops from where it started to where it'll make first contact, and it has to be done by using frametime
	// that way it matches perfectly with how it's done in the HL sdk.
	new count;
	new ignoreEnt;
	//new ballowner = GetBallData(BALL_OWNER);
	new ballid = GetBallData( BALL_ID );
	
	//hit entity might be worldspawn and we'd be stuck in the while loop.
	if(ballid == 0) 
		ballid = -1;
		
	while(VectorEqual(end, ret))
	{	
		//start from our end result of last pass
		start[0] = end[0];		start[1] = end[1];		start[2] = end[2];
		
		//compensate for gravity change.
		if(isPass && passtype == PASS_BOMB && vel[2] < 0.0)
		{
			gravity = serverGravity;
		}
		
		vel[2] -= gravity; //keep removing gravity
		
		end[0] += x; //move forward normally
		end[1] += y; //move forward normally
		end[2] += (vel[2] * frametime); //move down
		
		ignoreEnt = trace_line(ballid, start, end, ret);
		while(ignoreEnt > 0)
			ignoreEnt = trace_line(ignoreEnt, start, end, ret);
		
		count++;
		if(count > 1000)
			break;
	}
	
	orig[0] = ret[0];	orig[1] = ret[1];	orig[2] = ret[2];
	
	g_bLandEffect = true;
	
	return 1;
}
