//-----------------------------
// GAME BRAIN
//-----------------------------
public GameThink()
{
	if(!g_bGameStart)
		Handle_PreGame();
	else
	{
		Handle_UI();
		Handle_Ability();
	}
	
	Handle_Effects();
}

public GameTimer()
{
	if(!g_bGameStart)
		return;
			
	if(g_bGameTimerON)
		g_flGameTimer -= 0.5;

	FX_GameTime( floatround(g_flGameTimer) );
	
	if(g_flGameTimer <= 0.0)
	{
		g_bGameTimerON = false;
	}
}

StopClock()
{
	g_bGameTimerON = false;	
}

StartClock()
{
	g_bGameTimerON = true;	
}

bool:IsPlayClockOn()
{
	return g_bPlayClockOn;
}

StartPlayClock( ptime )
{
	g_bPlayClockOn = true;
	g_iPlayClock = ptime;
	remove_task( TASK_PLAYCLOCK );
	PlayClockCountdown( 0 );
}

StopPlayClock()
{
	FX_StatusIcon( g_iPlayClock, true );
	g_bPlayClockOn = false;	
}

public PlayClockCountdown( ptime )
{
	if(!g_bPlayClockOn || g_iGameMode != GAME_INPLAY)
		return;
		
	g_iPlayClock--;
	if(g_iPlayClock <= 0)
	{
		REFEREE( PENALTY_DELAYOFGAME );
		return;
	}
	
	set_task( 1.0, "PlayClockCountdown", TASK_PLAYCLOCK );	
}


bool:Handle_Scrimmage()
{
	//Did we pass line of scrimmage?
	if( !g_bPassedScrimmage && GetBallData( BALL_HOLDER ) != 0 )
	{
		if(g_iPlay == PLAY_KICKOFF || GetBallData(BALL_TEAM)!=GetBallData(BALL_START_TEAM))
			g_bPassedScrimmage = true;
		else
		{
			if(!IsPlayClockOn() && g_iGameMode == GAME_INPLAY)
			{
				g_bPassedScrimmage = (g_flCurrentBallYard > g_flCurrentYard);
			}
		}
	}	
}

public client_PreThink(id)
{
	Handle_Handoff(id);	//fb_ball.sma
	Handle_PlayerRules(id);
}

Handle_PlayerRules(id)
{
	Handle_Punt(id);
	
	new holder = GetBallData( BALL_HOLDER );
	if(holder != id)
		return;	
	
	if(g_iGameMode != GAME_INPLAY)
		return;
	
	new bool:playclock = IsPlayClockOn();
	if(playclock)
		return;		
	
	if(is_user_alive(id) && on_ground(id)) 
	{
		if(!g_bTouchedBack && g_flCurrentBallYard <= 0.0 )
		{
			new buttons = entity_get_int( id, EV_INT_button );
			if(buttons & IN_DUCK)
				REFEREE( KICK_TOUCHBACK );
		}
	}
}


public ThinkGrenade( grenade )
{
	Handle_Throw( grenade );
	
	//Remove the grenade entity immediately.
	remove_entity( grenade );
}

public Touch_Grenade( grenade, player )
{
	return PLUGIN_HANDLED;	
}

public Touch_OutofBound( out, entity )
{
	if(entity == GetBallData( BALL_ID ))
		g_iBounds[0] = OUTOFBOUND;
	else
	{
		g_iBounds[entity] = OUTOFBOUND;
		if(entity == GetBallData( BALL_HOLDER ))
			Handle_OutOfBound(entity);
	}	
}

public Touch_InBound( in, entity )
{
	if(entity == GetBallData( BALL_ID ))
		g_iBounds[0] = INBOUND;
	else
	{
		g_iBounds[entity] = INBOUND;
		if(entity == GetBallData( BALL_HOLDER ))
			Handle_InBound(entity);
				
	}
}

public Touch_EndZone( endzone, entity )
{
	if(entity == GetBallData( BALL_ID ))
		g_iBounds[0] = ENDZONE;
	else
	{
		g_iBounds[entity] = ENDZONE;
		if(entity == GetBallData( BALL_HOLDER ))
		{
			Handle_InBound(entity);
			Handle_TouchDown(endzone, entity);
		}
	}
}

public Touch_World( world, football )
{
	//Manage how our ball bounces.
	Football_Bounce( football ); 
	
	if( g_iGameMode != GAME_INPLAY || g_bPlayClockOn )
		return PLUGIN_CONTINUE;
		
	new bound = g_iBounds[0];
	new flytype = GetBallData( BALL_FLY );
	
	switch(flytype)
	{
		case FLY_PASS:	//First contact from a valid pass.
		{
			REFEREE( PASS_INCOMPLETE );
		}
		case FLY_FG:	//Field goal kick 
		{
			if(g_bExtraPoint)
				REFEREE( FAIL_EXTRAPOINT );
			else
				REFEREE( FAIL_FIELDGOAL );
			//REFEREE( FG_NOGOOD );
		}
		case FLY_PITCH, FLY_LATERAL, FLY_FUMBLE:	//Lateral toss.
		{
			if(bound == OUTOFBOUND)
			{
				if( g_flCurrentBallYard >= 100.0 )
				{
					ChangePossession( GetBallData( BALL_START_TEAM )==1?2:1 );
					REFEREE( TOUCHBACK );
				}
				else
					REFEREE( OUTOFBOUNDS );
			}
		}
		case FLY_PUNT, FLY_KICK: //Punt or Kickoff kick
		{
			//MarkBallYard( football );
			//new Float:origin[3];
			//entity_get_vector( football, EV_VEC_origin, origin );
			new Float:end = g_flCurrentBallYard;//GetYard( origin[1] );
			new Float:dist;
			new bool:isPunt = (flytype == FLY_PUNT);
			new startTeam = GetBallData(BALL_START_TEAM);
			dist = end - g_flStartYard[startTeam];
				
			//Did we kick backwards?
			if(dist < 0.0)
			{
				REFEREE( KICK_NEGATIVEYARD );
			}	
			else if(bound == OUTOFBOUND)
			{
				if(isPunt)
				{
					new kickDist = floatround(dist);
					new ballowner = GetBallData( BALL_OWNER );
					if(end >= 80.0 && end < 100.0)
						STATISTIC( ballowner, S_PUNT_IN20, 1 );
					STATISTIC( ballowner, S_PUNT_YARDS, kickDist );	
					
					if(GetGameStat( ballowner, S_PUNT_LONG ) < kickDist )
						STATISTIC( ballowner, S_PUNT_LONG, kickDist );
				}
				if(!g_bTouchedGround)
				{
					if(isPunt)
					{
						if( end >= 100.0 )
							REFEREE( PUNT_TOUCHBACK );
						else
							REFEREE( PUNT_OUTOFBOUNDS );
					}
					else
					{
						if( end >= 100.0 )
							REFEREE( KICK_TOUCHBACK );	
						else
							REFEREE( KICK_OUTOFBOUNDS );
					}
				}
				else
				{
					if( isPunt )
					{
						if( end >= 100.0 )
							REFEREE( PUNT_TOUCHBACK );
						else if( end > 0.0 )
							REFEREE( PUNT_ROLLED_OUT );
						else
							REFEREE( PUNT_OUTOFBOUNDS );
					}
					else
					{
						if( end >= 100.0 )
							REFEREE( KICK_TOUCHBACK );	
						else if(end > 0.0 )
							REFEREE( KICK_ROLLED_OUT );
					}
				}
			}
		}
	}
	
	//If we're inbound mark ground as touched.
	if( bound != OUTOFBOUND && !g_bTouchedGround)
		g_bTouchedGround = true;	

	//MarkBallYard(football);

	return PLUGIN_CONTINUE;
}

public Touch_Player( football, player )
{
	if(GetDefaultClass(player) == 0)
		return PLUGIN_CONTINUE;
		
	new flytype = GetBallData( BALL_FLY );
	if(flytype == FLY_DEAD)
		return PLUGIN_CONTINUE;
		
	//fb_ball.sma handles kicking rules
	if( Handle_Kick(player) )
		return PLUGIN_CONTINUE
		
	//Ball isn't touchable
	if(flytype == FLY_NONE || g_bPlayClockOn)
		return PLUGIN_CONTINUE;

	//Block us getting our own ball
	new owner = GetBallData( BALL_OWNER );
	new holder = GetBallData( BALL_HOLDER );
	if(holder == player)
		return PLUGIN_CONTINUE;
		
	new Float:tossedTime = get_gametime() - g_flFootballTossTime;
	
	if(owner == player && tossedTime <= 1.0)
		return PLUGIN_CONTINUE;
					
	new name[32];
	get_user_name(player, name, 31 );
	client_print( 0, print_chat, "%s got the ball!", name );
	
	if(g_iGameMode == GAME_INPLAY)
	{
		if(Football_Deflect( player ))
			return PLUGIN_CONTINUE;
			
		g_iCatchFlytype = flytype;
		
		//Manage rules for catching in and out of bounds.
		if(Handle_TouchPlayer(player))
			return PLUGIN_CONTINUE;
		
		g_bTouchedPlayer = true;
	}
	
	//Give Player ball.
	Football_Give(player, false);	
	
	set_task(0.7, "CurWeapon_FixViewModel", player );
	
	return PLUGIN_CONTINUE;
}

public Touch_FieldGoal( goal, football )
{
	if( g_iGameMode != GAME_INPLAY )
		return PLUGIN_CONTINUE;
		
	if(GetBallData( BALL_FLY ) != FLY_FG)
		return PLUGIN_CONTINUE;
		
	new dist = floatround(YARD_FG_POST - g_flCurrentYard);
	new owner = GetBallData(BALL_OWNER);
	new long = GetStat(owner, S_FG_LONG);
	
	STATISTIC( owner, S_FG_YARDS, dist );
	
	if(dist > 50)			STATISTIC( owner, S_FG_MADE_50, 1 );	
	else if(dist > 40)		STATISTIC( owner, S_FG_MADE_40, 1 );	
	else if(dist > 30)		STATISTIC( owner, S_FG_MADE_30, 1 );	
	else if(dist > 20)		STATISTIC( owner, S_FG_MADE_20, 1 );	
	else					STATISTIC( owner, S_FG_MADE_0, 1 );	
				
	if(dist > long)			STATISTIC( owner, S_FG_LONG, dist );
		
	if( g_bExtraPoint )		REFEREE( SCORE_EXTRAPOINT );
	else					REFEREE( SCORE_FIELDGOAL );	
		
	return PLUGIN_CONTINUE;
}

//Tracks when ball or player touches endzone for valid Touchdown.
Handle_TouchDown( goal, entity )
{
	if(g_iGameMode != GAME_INPLAY)
		return PLUGIN_CONTINUE;
	
	if(GetBallData(BALL_HOLDER) != entity && GetBallData(BALL_ID) != entity)
		return PLUGIN_CONTINUE;
		
	new bool:playClock = IsPlayClockOn()
	if(playClock)
		return PLUGIN_CONTINUE;
		
	new goalteam = entity_get_int( goal, EV_INT_team );
	new pteam = get_user_team(entity);
	new ballTeam = GetBallData( BALL_TEAM );
	
	if(goalteam == pteam && pteam == ballTeam)
	{
		//new startteam = GetBallData( BALL_START_TEAM );
		//client_print(0,print_chat,"extrapoint %i -- startteam equal playerteam %i", g_bExtraPoint?1:0, (startteam==playerteam)?1:0);
		
		if(g_bExtraPoint) 
		{
			//if(startteam == pteam)
			REFEREE( SCORE_TWOPOINT );
			//else
			//	REFEREE( FAIL_TWOPOINT );
		}
		else
		{
			REFEREE( SCORE_TOUCHDOWN );	
		}
		
	}
		
	return PLUGIN_CONTINUE;
}


bool:Handle_TouchPlayer(player)
{
	new bool:inbound = g_iBounds[player] != OUTOFBOUND;
	new flytype = GetBallData( BALL_FLY );
	new onground = on_ground(player);
	new ballTeam = GetBallData( BALL_TEAM );
	new startTeam = GetBallData( BALL_START_TEAM );
	new team = get_user_team(player);
	new bool:isStartTeam = (startTeam == team);
	
	switch(flytype)
	{
		case FLY_HANDOFF:
		{
			new class = GetClass(player);
			if(class == CLASS_RUNNINGBACK)
				g_flStamina[player] = 100.0;
			
			SetBallData( BALL_THROW_PLAYER, player );
			SetBallData( BALL_THROW_FLY, FLY_HANDOFF );	
			
			new ballowner = GetBallData( BALL_OWNER );
			Give( ballowner, "weapon_knife" );
		}
		case FLY_PUNT_HOLD:
		{
			g_iPlay = PLAY_STANDARD;	
		}
		case FLY_FUMBLE:
		{
			new fumbler = GetBallData( BALL_FUMBLE );
			if(ballTeam == team)
			{
				STATISTIC( fumbler, S_FUMBLE_RECOVER, 1 );	
			}
			else
			{
				SetBallData( BALL_THROW_PLAYER, 0 );
				SetBallData( BALL_THROW_FLY, 0 );	
			}
		}
		//Outofbounds
		case FLY_PASS:
		{
			if(!inbound && onground)
			{
				REFEREE( PASS_INCOMPLETE );
				return false;
			}
		}
		case FLY_FG:
		{
			g_iPlay = PLAY_STANDARD;
		}
		case FLY_FG_HOLD:
		{
			g_iPlay = PLAY_STANDARD;

		}
		case FLY_KICK:
		{
			new Float:origin[3];
			new Float:yard;
			entity_get_vector( player, EV_VEC_origin, origin );
			yard = GetYard( origin[1] );
				
			//Did we attempt to pickup our ball to short?
			if(!g_bTouchedPlayer && isStartTeam)
			{
				if(g_flCurrentBallYard - g_flStartYard[startTeam] <= MIN_KICK_DISTANCE)
				{
					REFEREE( KICK_SHORT );
					return true;
				}
			}
			
			//Did we catch ball out of bounds on a roll?
			if(!inbound && onground && g_bTouchedGround)
			{
				if( yard >= 100.0 )
					REFEREE( KICK_TOUCHBACK );	
				else
					REFEREE( KICK_ROLLED_OUT );
				return false;
			}
		}
		case FLY_PUNT:
		{
			new dist = floatround( g_flCurrentBallYard - g_flStartYard[startTeam] );
			new ballowner = GetBallData( BALL_OWNER );
			
			STATISTIC( ballowner, S_PUNT_YARDS, dist );
			
			if( !g_bTouchedPlayer )
			{
				//Did we catch punt 
				if(isStartTeam)
				{
					if(g_flCurrentBallYard >= 100.0)
						REFEREE( PUNT_TOUCHBACK );
					else
					{
						if(inbound)
						{
							REFEREE( PUNT_RETURNTOUCH );
							return true;
						}
						else
							REFEREE( PUNT_OUTOFBOUNDS );
					}
					return false;	
				} 	
			}
			
			if(!inbound && onground && g_bTouchedGround)
			{
				new Float:origin[3];
				entity_get_vector(player, EV_VEC_origin, origin );
				new Float:yard = GetYard( origin[1] );
				
				if( yard >= 100.0 )
					REFEREE( PUNT_TOUCHBACK );
				else if( yard >= 0.0 )
					REFEREE( PUNT_ROLLED_OUT );
				else
					REFEREE( PUNT_OUTOFBOUNDS );	
				return false;
			}
		}	
	}
	return false;
}

//Called when...
// Player catches ball.
// Player touches out of bound brush.
// Player dies.
Handle_OutOfBound(player, bool:isAlive=true)
{
	if(GetDefaultClass(player) == 0)
		return true;
	if(g_iGameMode != GAME_INPLAY || IsPlayClockOn())
		return true;
	
	new team = get_user_team(player);
	new startTeam = GetBallData( BALL_START_TEAM );
	//new possTeam = GetBallData( BALL_TEAM );
	new bool:onground = on_ground(player);
	new bool:isStartTeam = (startTeam == team);
	new owner = GetBallData( BALL_OWNER );
	new flytype = GetBallData( BALL_FLY );
	
	if(flytype == FLY_NONE || flytype == FLY_HANDOFF || flytype == FLY_PUNT_HOLD)
	{
		if(owner == player && (!isAlive || onground))
		{
			if(g_bExtraPoint)
			{
				REFEREE( FAIL_TWOPOINT );
				return true;
			}
			
			//Did we catch in the air and haven't landed or just did?
			switch(g_iCatchFlytype)
			{
				//We landed and were running
				case FLY_NONE:							REFEREE( OUTOFBOUNDS );		
				
				//Caught a pass in the air
				case FLY_PASS:							REFEREE( PASS_INCOMPLETE );
				
				case FLY_LATERAL:						REFEREE( OUTOFBOUNDS );
				
				//Caught a hand off in the air
				case FLY_HANDOFF:						REFEREE( OUTOFBOUNDS );
				
				//Caught a punt hold in the air
				case FLY_PUNT_HOLD:
				{
					if(isStartTeam)						REFEREE( OUTOFBOUNDS );
					else								REFEREE( PUNT_OUTOFBOUNDS );	
				}
				
				//Caught a kickoff in the air
				case FLY_KICK:
				{
					MarkBallYard(player);
					if( g_flCurrentBallYard >= 100.0 )	REFEREE( KICK_TOUCHBACK );	
					else								REFEREE( KICK_OUTOFBOUNDS );
				}
				
				//Caught a punt in the air.
				case FLY_PUNT:							REFEREE( PUNT_OUTOFBOUNDS );
			}
			return true;
		}
	}
	return false;
}


//This function considers the endzone in bound.
Handle_InBound(player, bool:isAlive=true)
{
	if(GetDefaultClass(player) == 0)
		return true;
		
	if(g_iGameMode != GAME_INPLAY || IsPlayClockOn())
		return true;
	
	new team = get_user_team(player);
	//new startTeam = GetBallData( BALL_START_TEAM );
	new ballTeam = GetBallData( BALL_TEAM );
	new bool:onground = on_ground(player);
	new bool:isBallTeam = (ballTeam == team);
	new holder = GetBallData(BALL_HOLDER);
	
	//Check Interception
	
	new flytype = GetBallData( BALL_FLY );
		
	if((!isAlive) || (onground && isAlive))
	{
		if(g_iCatchFlytype)
		{
			if(isBallTeam)
			{
				if(g_iCatchFlytype == FLY_KICK)
				{
					ChangePossession( team, 0 );
					g_iCatchFlytype = 0;
				}
			}
			else
			{
				if(g_iCatchFlytype==FLY_PASS || flytype==FLY_PASS)
				{
					//intType = 1;
						
					new ballPasser = GetBallData( BALL_THROW_PLAYER );
					STATISTIC( ballPasser, S_PASS_INTERCEPT, 1 );
					STATISTIC( player, S_INT, 1 );
						
					new passerFrags = get_user_frags(ballPasser);
					entity_set_float(ballPasser, EV_FL_frags, float(passerFrags-2));
					new frags = get_user_frags(player);
					entity_set_float(player, EV_FL_frags, float(frags+5));
						
					ChangePossession( team, 1 );
					g_iCatchFlytype = 0;
					
					
				}
				else if(g_iCatchFlytype == FLY_KICK)
				{
					ChangePossession( team, 0 );
					g_iCatchFlytype = 0;
				}
				else if(g_iCatchFlytype == FLY_HANDOFF || g_iCatchFlytype == FLY_FUMBLE || g_iCatchFlytype == FLY_LATERAL)
				{
					ChangePossession( team, 2 );	
					g_iCatchFlytype = 0;
				}
				else if(g_iCatchFlytype == FLY_PUNT || g_iCatchFlytype == FLY_PUNT_HOLD)
				{
					ChangePossession( team, 0 );
					g_iCatchFlytype = 0;	
				}
				else if(g_iCatchFlytype == FLY_FG || g_iCatchFlytype == FLY_FG_HOLD)
				{
					ChangePossession( team, 2 );
					g_iCatchFlytype = 0;	
				}
			}
		}
	}
		
	if(isAlive)
	{
		if(onground)
		{
			//Player Touched InBound (not endzone)
			if(!g_bTouchedBack && g_iBounds[player] == INBOUND)
				g_bTouchedBack = true;	
		
			//Player touched ground with ball.
			if(!g_bTouchedGround)
			{
				g_bTouchedGround = true;	
			}
			
			//Reset our CatchFlytype.
			if(g_iCatchFlytype)
				g_iCatchFlytype = 0;		
		}	
	}
	else		
	{
		//We were running ball and died.
		if(holder == player) 
		{	
			REFEREE( DOWN );
			return true;
		}
	}
		
	return false;
}

public Player_PreSpawn(id)
{
	Handle_PreSpawn(id);
	
}

public Player_PostSpawn(id)
{
	Handle_PostSpawn(id);
	//set_task( 0.1, "Player_PostSpawn_Post", id+35000 );
}

public Player_PostSpawn_Post(id)
{
	//id-=35000;
	//Handle_PostSpawn(id);
}

public Player_Killed(victim, killer, shouldgib )//, headshot, const weapon[])
{
	Handle_Death(killer, victim);
}
public Event_Damage(victim)
{
	if(!is_user_connected(victim) || !is_user_alive(victim))
		return PLUGIN_CONTINUE;
		
	new weaponID, attacker = get_user_attacker( victim, weaponID );
	if(!is_user_connected(attacker))
		return PLUGIN_CONTINUE;

	new damage = read_data(2);
	//new dmgtype = read_data(3);
	Handle_Damage( attacker, victim, damage );
	
	return PLUGIN_CONTINUE;
}

Handle_Death( killer, victim )
{
	new bool:inPlay = g_iGameMode == GAME_INPLAY;
	new holder = GetBallData( BALL_HOLDER );
		
	
	if(holder == victim)
	{
		
		new gamemode = g_iGameMode; //save here cause it might change.
		if(inPlay)
		{
			new pbounds = g_iBounds[holder];
			//We caught it out of bounds and are on ground, don't allow!
			if( pbounds == OUTOFBOUND ) 
				Handle_OutOfBound( victim, false );
			else
			{
				Handle_InBound( victim, false );
				new football = GetBallData( BALL_ID );
				MarkBallYard( football );
			}
		
			
				
			new class = GetClass(victim);
			
			if(class == CLASS_QUARTERBACK && (g_flCurrentBallYard <= g_flCurrentYard))
			{
				new totalYards = floatround( g_flCurrentYard - g_flCurrentBallYard );
				STATISTIC( victim, S_PASS_SACK, 1 );
				STATISTIC( victim, S_PASS_SACK_YARDS, totalYards );
				
				STATISTIC( killer, S_DEF_SACK, 1 );
			}
			else
			{
				STATISTIC( killer, S_DEF_TACKLE, 1 );
			}
			
			

			new flytype = GetBallData( BALL_FLY );
			
			Football_Fumble();	
			
			if(flytype == FLY_HANDOFF)
				Football_Fumble();	
			else
			{
				if(g_iGameMode == GAME_INPLAY)
					REFEREE( DOWN );
				else
					Football_Dead( g_flCurrentYard, false );
			}
				
			//SetBallData( BALL_HOLDER, victim );
		}
		else if(gamemode == GAME_NONE || !g_bGameStart)
			Football_Fumble();	
	}
	
	if(inPlay)
	{
		//Save position for respawn
		g_bPlayerDied[victim] = true;
		entity_get_vector(victim, EV_VEC_origin, g_vPlayerPos[victim] );
	}
	
	if( !IsPlayClockOn() )
		Respawn(victim, 1.5);
	else
		Respawn(victim, 1.5);
}

public FMCmdStart( id, uc_handle, randseed )
{
	if(!is_user_connected(id))
		return FMRES_IGNORED;
		
	new buttons = get_uc(uc_handle, UC_Buttons );
	new bool:hasball = (GetBallData(BALL_HOLDER) == id);//PlayerHasBall(id);
	
	
	if(g_bPlayClockOn && g_iPlayClock < 9 && g_iGameMode == GAME_INPLAY )
	{
		new class = GetClass(id);
		new flytype = GetBallData( BALL_FLY );
		if( (hasball && class == CLASS_QUARTERBACK && g_iPlay == PLAY_STANDARD) || 
				(class == CLASS_KICKER && ((flytype == FLY_FG_HOLD && g_iPlay == PLAY_FIELDGOAL) || (flytype == FLY_PUNT_HOLD && g_iPlay == PLAY_PUNT))) )
		{
			//new startTeam = GetBallData( BALL_START_TEAM );
			
			new buttonSet = buttons & ~(IN_FORWARD | IN_MOVELEFT | IN_MOVERIGHT | IN_BACK | IN_ATTACK);
			set_uc(uc_handle, UC_Buttons, buttonSet );
			
			if(g_iHutCount > 0)
			{
				if( !(buttons & IN_ATTACK2) && (g_iButtons[id] & IN_ATTACK2))
					Hut();
			}
		}
	}
	else
	{
		
		Handle_AbilityCmdStart( id, buttons );
		set_uc(uc_handle, UC_Buttons, buttons );
		
		if(hasball)
		{
			//Handle throwing forward exploits when not suppose to.
			new weapon = get_user_weapon(id);
			new flytype = GetBallData( BALL_FLY );
			if( g_bPassedScrimmage && flytype == FLY_NONE && weapon == CSW_HEGRENADE)
			{
				new dir = PlayerDirection(id);
				if(dir == _FORWARD)
				{
					if(!(buttons & IN_ATTACK) && (g_iButtons[id] & IN_ATTACK) )
					{
						Football_Give(id);
						buttons &= ~IN_ATTACK;
					}
					else if( (buttons & IN_ATTACK) && !(g_iButtons[id] & IN_ATTACK) )
					{
						set_uc(uc_handle, UC_Buttons, (buttons & ~IN_ATTACK) );
						buttons &= ~IN_ATTACK;
					}
				}	
			}
		}
	}
		
	g_iButtons[id] = buttons;
	
	return FMRES_IGNORED;
}

Hut()
{
	client_print(0,print_chat, "HUT!" );
	g_iHutCount--;
				
	if(g_iHutCount <= 0)
	{
		UnfreezeAll();
			
		if(!g_bExtraPoint)
			StartClock();
		StopPlayClock();
	}
}

public cmdTurbo(id)
{
	
	if(Ability_Activate( id, A_TURBO_QUICK ))//g_iSprint[id])
	{
		
		//g_iSprint[id] = 1;
		Ability_SetDelay( id, A_TURBO_QUICK, get_gametime() );
		//g_flTurboDelay[id] = get_gametime();
	}	
	return PLUGIN_HANDLED;
}