/*---------------------------------------------------------------------------

	Rules
	
	Purpose:
		This file handles all the rules for statistics and play stopping events (ie. Out of bounds, Touchdown).
		It also manages some sensitive data for determining these outcomes. 

---------------------------------------------------------------------------*/

REFEREE( event )
{
	//Game Events
	if(event == GAME_START)	{
		BeginQuarter(); return;
	}
	else if(event == GAME_END_QUARTER)	{
		EndQuarter(); return;	
	}
	
	g_iGameMode = GAME_PENALTY;
	//g_iLastPlay = g_iPlay;
	g_iLastEvent = event;
	
	//new owner = GetBallData( BALL_OWNER );
	new holder = GetBallData( BALL_HOLDER );
	new ballTeam = GetBallData( BALL_TEAM );
	new startTeam = GetBallData( BALL_START_TEAM );
	//new oteam = ballTeam == 1 ? 2 : 1;
	new qb, kicker;//, safety;
	qb = GetCurrentSpecial( ballTeam, CLASS_QUARTERBACK );
	kicker = GetCurrentSpecial( ballTeam, CLASS_KICKER );
	//safety = GetCurrentSpecial( oteam, CLASS_SAFETY );
	
	//GetCurrentSpecials( qb, kicker, safety );
	
	//Just in case it happens.
	if(g_flCurrentBallYard >= 100.0)
		g_flCurrentBallYard = 99.99;
		
	//Begin the play. (Done up here so we can remove it if we need too.)
	set_task( RESTART_STATE_TIME, "BeginPlay", TASK_BEGINPLAY );
		
	//Stop clock unless down inside.
	if(event > DOWN)
		StopClock();	
	
	if(g_bExtraPoint && event < PENALTY_EVENT)
	{
		g_bExtraPoint = false;
		if( event != FAIL_EXTRAPOINT && 
			event != SCORE_EXTRAPOINT && 
			event != SCORE_TWOPOINT)
		{
			remove_task( TASK_BEGINPLAY );
			REFEREE( FAIL_TWOPOINT );
			return;
		}
		
		if( event == SCORE_TWOPOINT && startTeam != ballTeam )
		{
			SetBallData( BALL_TEAM, startTeam );
		}
	}
	
	switch( event )
	{
		//----------------------------------------
		//Misc Events
		case DOWN:			//Down in bound
		{
			new yardage[32];
			GetText_YardLine( floatround(g_flCurrentBallYard), yardage );
			
			new szMsg[128];
			if(ballTeam == startTeam && qb == holder && g_flCurrentBallYard < g_flCurrentYard && g_iPlay == PLAY_STANDARD)
				format( szMsg, 127, "Sacked at %s.", yardage );
			else
				format( szMsg, 127, "Down at %s.", yardage );
				
			if(CheckDown())
			{
				//Manage switching to new QB or Kicker or Safety.
				Class_EndPlay();
				return;
			}
			
			
			format( g_HudMsg3, 127, "%s", szMsg );
			
			g_iPlay = PLAY_STANDARD;
		}
		case TOUCHBACK:		//Down at endzone/outofbound
		{
			format( g_HudMsg3, 127, "Touchback");
			g_flCurrentYard = YARD_TOUCHBACK;
			FirstDown();
			
			g_iPlay = PLAY_STANDARD;
		}
		case OUTOFBOUNDS: 	//Down out of bounds
		{
			
			if(CheckDown())
			{
				//Manage switching to new QB or Kicker or Safety.
				Class_EndPlay();
				return;
			}
			
			new yardage[32];
			GetText_YardLine( floatround(g_flCurrentBallYard), yardage );
			format( g_HudMsg3, 127, "Out of bounds at %s.", yardage );
			
			g_iPlay = PLAY_STANDARD;
		}
		case TWOMINUTE:		//Two minute warning
		{
		}
		
		//----------------------------------------
		//Scoring Events
		case SCORE_FIELDGOAL:		//3 Points
		{	
			format( g_HudMsg3, 127, "IT'S GOOD!");
			g_iScore[ballTeam] += 3;
			g_iPlay = PLAY_KICKOFF;
		}
		case SCORE_TOUCHDOWN:		//6 points
		{
			SetBallData( BALL_START_TEAM, ballTeam );
			format( g_HudMsg3, 127, "TOUCHDOWN!^n%s!", ballTeam == T ? "Terrorists":"Counter-Terrorists" );
			Stat_EndPlay();
			
			g_iScore[ballTeam] += 6;
			
			g_iGameMode = GAME_EXTRA;
			g_bExtraPoint = true;
			StripAll();
			remove_task( TASK_BEGINPLAY );
			BeginVote( VOTE_EXTRAPOINT, ballTeam );
		}
		case SCORE_EXTRAPOINT:		//1 point
		{
			format( g_HudMsg3, 127, "IT'S GOOD!^n%s!", ballTeam == T ? "Terrorists":"Counter-Terrorists" );
			g_iScore[ballTeam] += 1;
			g_iPlay = PLAY_KICKOFF;
		}
		case SCORE_TWOPOINT:		//2 points
		{
			format( g_HudMsg3, 127, "TWO POINTS!^n%s!", ballTeam == T ? "Terrorists":"Counter-Terrorists" );
			Stat_EndPlay();
			
			g_iScore[ballTeam] += 2;
			g_iPlay = PLAY_KICKOFF;
		}
		case SCORE_SAFETY:			//2 points
		{
			format( g_HudMsg3, 127, "SAFETY!");
			Stat_EndPlay();
			
			g_iScore[ballTeam==1?2:1] += 2;
			g_iPlay = PLAY_KICKOFF;
			
		}
		
		//----------------------------------------
		//Failed Attempt Events
		case FAIL_FIELDGOAL:	//Failed 3 pt FG attempt
		{
			format( g_HudMsg3, 127, "No Good.");
			ChangePossession( startTeam == 1 ? 2 : 1 );
			g_iPlay = PLAY_STANDARD;
		}
		case FAIL_EXTRAPOINT:	//Failed extra pt FG
		{
			format( g_HudMsg3, 127, "No Good.");
			ChangePossession( startTeam );
			g_iPlay = PLAY_KICKOFF;
		}
		case FAIL_TWOPOINT:		//Failed extra pt 2 pt conversion
		{
			format( g_HudMsg3, 127, "No Good.");
			ChangePossession( startTeam );
			g_iPlay = PLAY_KICKOFF;
		}
		
		//----------------------------------------
		//Kick Events
		case KICK_NEGATIVEYARD:	//Kicked in the opposite direction
		{
			new yardage[32];
			GetText_YardLine( floatround(YARD_PUNT_PENALTY), yardage );
			format( g_HudMsg3, 127, "Kicked negative yards. ^nSetting to %s.", yardage);
			
			ChangePossession( startTeam == 1 ? 2 : 1 );
			g_flCurrentYard = YARD_PUNT_PENALTY;
			FirstDown();
			
			g_iPlay = PLAY_STANDARD;
		}
		case KICK_SHORT:		//Touched less than X yards from kicking yard.
		{
			
			ChangePossession( startTeam == 1 ? 2 : 1 );
			g_flCurrentYard = 100-g_flCurrentBallYard;
			FirstDown();
			
			g_iPlay = PLAY_STANDARD;
			
			new yardage[32];
			GetText_YardLine( floatround(g_flCurrentYard), yardage );
			format( g_HudMsg3, 127, "Short kick. ^nSetting to %s.", yardage);
		}
		case KICK_OUTOFBOUNDS:	//Kick landed out of bounds.
		{
			new yardage[32];
			GetText_YardLine( floatround(YARD_PUNT_PENALTY), yardage );
			format( g_HudMsg3, 127, "Kick landed out of bounds. ^nSetting to %s.", yardage );
			STATISTIC( kicker, S_KICK_OUTOFBOUND, 1 );
			
			ChangePossession( startTeam == 1 ? 2 : 1 );
			g_flCurrentYard = YARD_PUNT_PENALTY;
			FirstDown();
			
			g_iPlay = PLAY_STANDARD;
		}
		case KICK_TOUCHBACK:	//Team kneeled/ran out in endzone
		{
			format( g_HudMsg3, 127, "Touchback.");
			STATISTIC( kicker, S_KICK_TOUCHBACK, 1 );
			
			ChangePossession( startTeam == 1 ? 2 : 1 );
			g_flCurrentYard = YARD_TOUCHBACK;
			FirstDown();
			
			g_iPlay = PLAY_STANDARD;
			//FirstDown();
		}
		case KICK_ROLLED_OUT:	//Ball rolled out of bounds
		{
			format( g_HudMsg3, 127, "Kick rolled out of bounds.");
			//g_flCurrentYard = g_flCurrentBallYard;
			ChangePossession( startTeam == 1 ? 2 : 1 );
	
			g_flCurrentYard = YARD_PUNT_PENALTY;
			FirstDown();
			
			g_iPlay = PLAY_STANDARD;
		}
		
		//----------------------------------------
		//Punt Events
		case PUNT_NEGATIVEYARD:  //Kicked opposite direction
		{
			format( g_HudMsg3, 127, "Punted negative yards.");
		}
		case PUNT_RETURNTOUCH:	//Return team touched ball
		{
			new yardage[32];
			GetText_YardLine( floatround(g_flCurrentBallYard), yardage );
			format( g_HudMsg3, 127, "Punting team touched ball at %s.", yardage);
			g_flCurrentYard = g_flCurrentBallYard;
			ChangePossession( startTeam == 1 ? 2 : 1 );
			
			g_iPlay = PLAY_STANDARD;
		}
		case PUNT_OUTOFBOUNDS:	//Ball landed out of bounds
		{
			format( g_HudMsg3, 127, "Punt landed out of bounds.");
			STATISTIC( kicker, S_PUNT_OUTOFBOUND, 1 );
			
			g_flCurrentYard = g_flCurrentBallYard;
			ChangePossession( startTeam == 1 ? 2 : 1 );

			g_iPlay = PLAY_STANDARD;
		}
		case PUNT_TOUCHBACK:	//Ball landed out of bounds at endzone. (several ways for this to happen)
		{
			format( g_HudMsg3, 127, "Touchback.");
			
			STATISTIC( kicker, S_PUNT_TOUCHBACK, 1 );

			ChangePossession( startTeam == 1 ? 2 : 1 );
			g_flCurrentYard = YARD_TOUCHBACK;
			FirstDown();
			
			g_iPlay = PLAY_STANDARD;
		}
		case PUNT_ROLLED_OUT:	//Ball rolled out of bounds
		{
			new yardage[32];
			GetText_YardLine( floatround(g_flCurrentBallYard), yardage );
			format( g_HudMsg3, 127, "Punt rolled out at the %s yard line.", yardage );
			g_flCurrentYard = g_flCurrentBallYard;
			ChangePossession( startTeam == 1 ? 2 : 1 );
			g_iPlay = PLAY_STANDARD;
		}
		
		//----------------------------------------
		//Pass Events
		case PASS_CATCH_OUTOFBOUNDS:	//Pass caught out of bounds
		{
			format( g_HudMsg3, 127, "Passed caught out of bounds." );
			NextDown();
		}
		case PASS_INCOMPLETE:			//Pass missed by all.
		{
			format( g_HudMsg3, 127, "Pass Incomplete.");
			NextDown();
		}
		case PASS_OUTOFBOUNDS:			//Pass landed out of bounds.
		{
			format( g_HudMsg3, 127, "Passed out of bounds." );
			NextDown();
		}
		
		//----------------------------------------
		//Penalties
		case PENALTY_DELAYOFGAME:	//Team took to long to snap ball.
		{
			format( g_HudMsg3, 127, "Delay of Game." );
			UnfreezeAll();
		}
		case PENALTY_PASSINT:		//defense knifed receiver with ball in air headed his way
		{
		}
		case PENALTY_OFFSIDES:		//Player on wrong side of scrimmage after 2nd hut
		{
		}
	}
	
	//if(event != SCORE_TOUCHDOWN )
	if(g_iPlay != PLAY_KICKOFF)
	{
		g_bPaintLine = true;
	}
	
	Football_Dead( g_flCurrentYard );
	
	if(g_bExtraPoint)
	{
		g_flCurrentYard = YARD_EXTRAPOINTS;
		g_flTargetYard = YARD_GOALLINE;
	}
	else if(g_iPlay == PLAY_KICKOFF)
		g_flCurrentYard = YARD_KICKOFF;
	
	//Manage switching to new QB or Kicker or Safety.
	Class_EndPlay();
				
	//new minutes = floatround(g_flGameTimer/60.0, floatround_ceil );
	//new seconds = floatround(g_flGameTimer) % 60;
	
	//client_print( 0, print_chat, "Referee Call : %s", g_szRefereeCall[event] );
	//client_print( 0, print_chat, "Current Yard : %.2f", g_flCurrentYard );
	//log_amx( "------------------------------------------");
	//log_amx( "Current Play: %s", g_szPlay[g_iLastPlay] );
	//log_amx( "Clock Time: %i min %i sec", minutes, seconds );
	//log_amx( "Referee Call : %s", g_szRefereeCall[event] );
	//log_amx( "Current Yard: %i", floatround( g_flCurrentYard ) );
	//log_amx( "Next Play: %s", g_szPlay[g_iPlay] );
	//log_amx( "------------------------------------------");
	
	//Should we end the game here?
	if(ShouldEndQuarter())
	{
		REFEREE( GAME_END_QUARTER );
		return;
	}
}


public BeginQuarter()
{
	g_iQuarter++;
	
	if(g_iQuarter > 4)
		client_print(0,print_chat, "Sudden Death = %i", g_iQuarter-4 );
	else
		client_print(0,print_chat, "Quarter = %i", g_iQuarter );
		
	g_flGameTimer = get_pcvar_float( P_GAMETIME ) * 60;
	g_bGameTimerON = false;
	
	//Manage 1st and 3rd quarters and OverTime
	if(g_iQuarter == 1)
	{
		g_bGameStart = true;
		g_iFirstKickoffTeam = GetBallData( BALL_TEAM );
		
		g_iScore[T] = 0;
		g_iScore[CT] = 0;
		
		g_flCurrentYard = YARD_KICKOFF;
		BeginPlay( PLAY_KICKOFF );
	}
	else if(g_iQuarter == 3)
	{
		new oteam = g_iFirstKickoffTeam == 1 ? 2 : 1;
		
		ChangePossession( oteam );
		g_flCurrentYard = YARD_KICKOFF;
		BeginPlay( PLAY_KICKOFF );
	}
	else if(g_iQuarter > 4)
	{
		//Double over time switch kickoff team.
		if(g_iQuarter % 2 == 0)
			ChangePossession( g_iFirstKickoffTeam == 1 ? 2 : 1 );
		else
			ChangePossession( g_iFirstKickoffTeam );
			
		g_flCurrentYard = YARD_KICKOFF;
		BeginPlay( PLAY_KICKOFF );
	}
	else
	{
		BeginPlay( TASK_BEGINPLAY );
	}
}
		
EndQuarter()
{
	remove_task(TASK_PLAYCLOCK);
	remove_task(TASK_BEGINPLAY);
	remove_task(VOTE_EXTRAPOINT);
	remove_task(VOTE_FOURTHDOWN);
	
	if(g_iQuarter >= 4)
	{
		if(g_iScore[T] == g_iScore[CT])
			OverTime();
		else
			EndGame();		
	}
	else if(g_iQuarter == 2)
	{
		HalfTime();
	}
	else
	{
		format( g_HudMsg3, 127, "End of the %i%s quarter.", g_iQuarter, g_szSuffix[g_iQuarter] );
		set_task( RESTART_STATE_TIME*2, "BeginQuarter", TASK_BEGIN_QUARTER );
	}
}

HalfTime()
{
	format( g_HudMsg3, 127, "HALF TIME", g_iQuarter, g_szSuffix[g_iQuarter] );
	
	g_bExtraPoint = false;
	new oteam = g_iFirstKickoffTeam == 1 ? 2 : 1;
	client_print(0,print_center, "%s will receive the ball.", oteam==1?"Terrorist":"Counter-Terrorist" );	
	
	//Tournament Mode: Activate Ready System
	//Public Mode: Begin Countdown from 30 (time to look at stats?)
	set_task( RESTART_STATE_TIME*2, "BeginQuarter", TASK_BEGIN_QUARTER );
}

OverTime()
{
	if(g_iQuarter == 4)
		format( g_HudMsg3, 127, "Game Tied.^nSudden Death up next.", g_iQuarter, g_szSuffix[g_iQuarter] );
	else
		format( g_HudMsg3, 127, "Sudden Death^nRound %i", g_iQuarter-3 );
		
	g_iFirstKickoffTeam = random_num(T,CT);
	client_print(0,print_center, "%s will receive the ball.", g_iFirstKickoffTeam==1?"Terrorist":"Counter-Terrorist" );
	
	set_task( RESTART_STATE_TIME, "BeginQuarter", TASK_BEGIN_QUARTER );
}

EndGame()
{
	if(g_iScore[T] > g_iScore[CT] || g_iScore[CT] > g_iScore[T])
	{
		format( g_HudMsg3, 127, "Game Ended.^n%s wins!", g_iScore[T] > g_iScore[CT] ? "Terrorists" : "Counter-Terrorists" );
		client_cmd(0, "spk fjam/nfl.wav");
	}
	client_print( 0,print_chat, "Game Ended." );

	ClearTasks();
	
	g_bExtraPoint = false;
	g_iGameMode = GAME_NONE;
	
	if(!g_bTournamentMode)
	{
		set_task( TASKTIME_RANDOMTEAMS, "randomize_teams", TASK_RANDOMTEAMS );
		set_task( TASKTIME_RESETGAME, "ResetGame", TASK_RESETGAME );
	}
	else
		set_task( TASKTIME_RESETGAME_TOURNAMENT, "ResetGame", TASK_RESETGAME );
	//new iReturn;
	//ExecuteForward( FORWARD_TOURNAMENTMODE, iReturn, 1 ); //1 = RESET
}

public ResetGame()
{
	g_bGameStart = false;
	g_iQuarter = 0;
}

public BeginPlay( playtype )
{	
	
	remove_task( TASK_BEGINPLAY );
	if(playtype != TASK_BEGINPLAY)
		g_iPlay = playtype;
		
	//log_amx( "BEGIN PLAY playtype(%i) g_iPlay(%i)", playtype, g_iPlay );
	g_iGameMode = GAME_INPLAY;
	
	//UnfreezeAll();
	//new iReturn;
	//ExecuteForward( FORWARD_BEGIN_PLAY, iReturn, playtype );
	Football_Clear( false );
	//Football_Dead(g_flCurrentYard, false);
	
	new ballTeam = GetBallData( BALL_TEAM );
	//new Float:yard = g_flCurrentYard;
	
	SetBallData( BALL_HOLDER, 0 );
	SetBallData( BALL_OWNER, 0 );
	SetBallData( BALL_START_TEAM, ballTeam );
	SetBallData( BALL_DIRECTION, (ballTeam == T) ? _BACK : _FORWARD );
		
	//Begin the play clock for delay of game.
	StartPlayClock( 10 );
	
	//Clear any reserved positions.
	ClearReserved();
	
	Football_Dead( g_flCurrentYard, false );
		
	Class_StartPlay();
	
	//Respawn all players.	
	Respawn(0);
	
	//Reset Variables
	if(playtype != PLAY_KICKOFF)
	{
		g_bPaintLine = true;
		g_bPassedScrimmage = false;
		g_iHutCount = 2;
	}
		
	g_iCatchFlytype = 0;
	g_bTouchedGround = false;
	g_bTouchedPlayer = false;	
	if(g_iPlay != PLAY_STANDARD && g_iPlay != PLAY_PUNT)
		g_bTouchedBack = false;
	else
		g_bTouchedBack = true;

	g_HudMsg3[0] = 0;
	set_hudmessage(20, 255, 20, -1.0, 0.4, 0, 1.0, 1.5, 0.1, 0.1, CHANNEL3)
	show_hudmessage(0,"")
	set_hudmessage(20, 255, 20, -1.0, 0.4, 0, 1.0, 1.5, 0.1, 0.1, CHANNEL4)
	show_hudmessage(0,"")
}

//Turnover: 0 = none, 1 = Interception, 2 = Turnover
ChangePossession( team, turnover=0 )
{
	new ballteam = GetBallData( BALL_TEAM );
	if( ballteam != 0 && team != ballteam )
	{
		g_flCurrentYard = 100.0 - g_flCurrentYard;	
		
		if(turnover == 1)
		{
			SetBallData( BALL_TEAM, team );
			MarkBallYard( GetBallData(BALL_ID) );
			g_flCurrentYard = g_flCurrentBallYard;//100.0 - g_flCurrentYard;	
		}
	}
		
	SetBallData( BALL_TEAM, team );
	SetBallData( BALL_DIRECTION, (ballteam == T) ? _BACK : _FORWARD );
	
	FirstDown();

	g_HudMsg4[0] = 0;
	if(turnover == 1)
		format( g_HudMsg4, 63, "INTERCEPTION!" );
	else if(turnover == 2)
		format( g_HudMsg4, 63, "TURNOVER!" );

	if(turnover > 0)
	{
		g_bTouchedBack = false;
		g_bPassedScrimmage = true;
		g_bPaintLine = false;
	}
		
	if(g_HudMsg4[0])
	{
		client_print(0,print_chat, "%s", g_HudMsg4 );
		g_MsgTime = get_gametime() + 0.5;
		set_hudmessage(255, 0, 20, -1.0, 0.3, 0, 1.0, 1.5, 0.1, 0.1, CHANNEL4)
		show_hudmessage(0, "%s", g_HudMsg4)
	}
}

NextDown()
{
	if(g_flCurrentYard >= g_flTargetYard)
	{
		new bool:oldTime = g_bGameTimerON;
		FirstDown();
		g_bGameTimerON = oldTime;
		return;	
	}
	
	g_iDown++;
	
	if(g_iDown == 4)
	{
		remove_task(TASK_BEGINPLAY);
		BeginVote( VOTE_FOURTHDOWN, GetBallData( BALL_TEAM ) );	
	}
	else if(g_iDown > 4)
	{
		ChangePossession( (GetBallData( BALL_TEAM ) == 1 ? 2 : 1) );	
		
		format(g_HudMsg3, 127, "^nEnd of 4th down!" );
	}	
}

FirstDown()
{
	if(g_iGameMode != GAME_INPLAY)
		StopClock();
	g_iDown = 1;
	
	//g_iPlay = PLAY_STANDARD;
	
	//Add yards for next first down location.
	g_flTargetYard = g_flCurrentYard + FIRST_DOWN_YARDS;
	
	//Can't go passed 100, thats a touchdown.
	if(g_flTargetYard > 100.0)
		g_flTargetYard = 100.0;
}	

//Checks for when we are tackled or ran out of bounds.
bool:CheckDown()
{
	if(g_flCurrentBallYard <= 0.0)
	{
	 	if(g_bTouchedBack) {
			REFEREE( SCORE_SAFETY );
			return true;
		}
	 	else {
	 		REFEREE( TOUCHBACK );
	 		return true;
			}
	}
	else if(g_bExtraPoint) {
		REFEREE( FAIL_TWOPOINT );
		return true;
	}	
				
	Stat_EndPlay();
	
	g_flCurrentYard = g_flCurrentBallYard;
	
	new ballTeam = GetBallData(BALL_TEAM);
	new startTeam = GetBallData(BALL_START_TEAM);

	if(ballTeam != startTeam)
		FirstDown();
	else if(g_iPlay == PLAY_KICKOFF)
		FirstDown();
	else
		NextDown();
	return false;					
}

bool:ShouldEndQuarter()
{
	if(g_iQuarter > 4 && g_iScore[T] != g_iScore[CT] )
	{
		client_print(0,print_chat, "Ending Sudden Death #%i", g_iQuarter-4 );
		return true;
	}
		
	if(g_flGameTimer <= 0.0 && !g_bExtraPoint)
		return true;
		
	return false;
	/*
	new id, cnt, team;
	for(id=1;id<=g_iMaxplayers;id++)
		if(is_user_connected(id) && g_iPlayerClass[id] != 0)
			cnt++;
	
	if(cnt < get_pcvar_num(P_PLAYERSNEEDED))
		return true;
	
	return false;
	*/	
}